'use strict';

exports.__esModule = true;

var _partial2 = require('lodash/partial');

var _partial3 = _interopRequireDefault(_partial2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _axes = require('../../../lib/canvas/axes');

var _props2 = require('./props');

var _enhance = require('./enhance');

var _enhance2 = _interopRequireDefault(_enhance);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var renderRect = function renderRect(ctx, _ref) {
    var x = _ref.x,
        y = _ref.y,
        width = _ref.width,
        height = _ref.height,
        color = _ref.color,
        labelTextColor = _ref.labelTextColor,
        value = _ref.value;

    ctx.fillStyle = color;
    ctx.fillRect(x - width / 2, y - height / 2, width, height);

    ctx.fillStyle = labelTextColor;
    ctx.fillText(value, x, y);
};

var renderCircle = function renderCircle(ctx, _ref2) {
    var x = _ref2.x,
        y = _ref2.y,
        width = _ref2.width,
        height = _ref2.height,
        color = _ref2.color,
        labelTextColor = _ref2.labelTextColor,
        value = _ref2.value;

    var radius = Math.min(width, height) / 2;

    ctx.fillStyle = color;
    ctx.beginPath();
    ctx.arc(x, y, radius, 0, 2 * Math.PI);
    ctx.fill();

    ctx.fillStyle = labelTextColor;
    ctx.fillText(value, x, y);
};

var computeNodes = function computeNodes(_ref3) {
    var data = _ref3.data,
        keys = _ref3.keys,
        getIndex = _ref3.getIndex,
        xScale = _ref3.xScale,
        yScale = _ref3.yScale,
        sizeScale = _ref3.sizeScale,
        cellWidth = _ref3.cellWidth,
        cellHeight = _ref3.cellHeight,
        colorScale = _ref3.colorScale,
        getLabelTextColor = _ref3.getLabelTextColor;
    return data.reduce(function (acc, d) {
        keys.forEach(function (key) {
            var width = sizeScale ? Math.min(sizeScale(d[key]) * cellWidth, cellWidth) : cellWidth;
            var height = sizeScale ? Math.min(sizeScale(d[key]) * cellHeight, cellHeight) : cellHeight;

            var node = {
                xKey: key,
                yKey: getIndex(d),
                x: xScale(key),
                y: yScale(getIndex(d)),
                width: width,
                height: height,
                value: d[key],
                color: colorScale(d[key])
            };

            acc.push(Object.assign(node, {
                labelTextColor: getLabelTextColor(node)
            }));
        });

        return acc;
    }, []);
};

var HeatMapCanvas = function (_Component) {
    _inherits(HeatMapCanvas, _Component);

    function HeatMapCanvas() {
        _classCallCheck(this, HeatMapCanvas);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    HeatMapCanvas.prototype.componentDidMount = function componentDidMount() {
        this.ctx = this.surface.getContext('2d');
        this.draw(this.props);
    };

    HeatMapCanvas.prototype.shouldComponentUpdate = function shouldComponentUpdate(props) {
        this.draw(props);
        return false;
    };

    HeatMapCanvas.prototype.draw = function draw(props) {
        var width = props.width,
            height = props.height,
            outerWidth = props.outerWidth,
            outerHeight = props.outerHeight,
            margin = props.margin,
            offsetX = props.offsetX,
            offsetY = props.offsetY,
            xScale = props.xScale,
            yScale = props.yScale,
            cellShape = props.cellShape;


        this.surface.width = outerWidth;
        this.surface.height = outerHeight;

        var renderNode = void 0;
        if (cellShape === 'rect') {
            renderNode = (0, _partial3.default)(renderRect, this.ctx);
        } else {
            renderNode = (0, _partial3.default)(renderCircle, this.ctx);
        }

        var nodes = computeNodes(props);

        this.ctx.clearRect(0, 0, outerWidth, outerHeight);
        this.ctx.translate(margin.left + offsetX, margin.top + offsetY);

        (0, _axes.renderAxes)(this.ctx, {
            xScale: xScale,
            yScale: yScale,
            width: width - offsetX * 2,
            height: height - offsetY * 2,
            top: props.axisTop,
            right: props.axisRight,
            bottom: props.axisBottom,
            left: props.axisLeft
        });

        this.ctx.textAlign = 'center';
        this.ctx.textBaseline = 'middle';

        nodes.forEach(renderNode);
    };

    HeatMapCanvas.prototype.render = function render() {
        var _this2 = this;

        var _props = this.props,
            outerWidth = _props.outerWidth,
            outerHeight = _props.outerHeight;


        return _react2.default.createElement('canvas', {
            ref: function ref(surface) {
                _this2.surface = surface;
            },
            width: outerWidth,
            height: outerHeight
        });
    };

    return HeatMapCanvas;
}(_react.Component);

HeatMapCanvas.propTypes = _props2.HeatMapPropTypes;

exports.default = (0, _enhance2.default)(HeatMapCanvas);